import { AnyFn, AssertKeyOf, Promisify, ReplaceNever } from '@rhao/types-base';
import { HookTaskCaller } from './callers';
import { HookContext } from './Context';
import { DeprecatedHook, HookCallback, HookKeys, NestedHooks } from './types';
import { noop } from './utils';
export type InferCallback<HT, HN extends HookKeys<HT>> = HT[AssertKeyOf<HN, HT>] extends HookCallback ? ReplaceNever<HT[AssertKeyOf<HN, HT>], AnyFn> : AnyFn;
export type InferCallbackParams<HT, HN extends HookKeys<HT>> = Parameters<InferCallback<HT, HN>>;
export type InferCallbackReturn<HT, HN extends HookKeys<HT>> = ReturnType<InferCallback<HT, HN>>;
export declare class Hookable<HooksT extends Record<string, any> = Record<string, HookCallback>, HookNameT extends HookKeys<HooksT> = HookKeys<HooksT>> {
    private _hooks;
    private _before?;
    private _after?;
    private _deprecatedHooks;
    private _deprecatedMessages?;
    constructor();
    hook: <NameT extends HookNameT>(name: NameT, fn: InferCallback<HooksT, NameT>, options?: {
        allowDeprecated?: boolean;
    }) => typeof noop;
    hookOnce: <NameT extends HookNameT>(name: NameT, fn: InferCallback<HooksT, NameT>) => typeof noop;
    removeHook: <NameT extends HookNameT>(name: NameT, fn: InferCallback<HooksT, NameT>) => void;
    deprecateHook: <NameT extends HookNameT>(name: NameT, deprecated: HookKeys<HooksT> | DeprecatedHook<HooksT>) => void;
    deprecateHooks: (deprecatedHooks: Partial<Record<HookNameT, DeprecatedHook<HooksT>>>) => void;
    /**
     * 添加 hooks
     */
    addHooks: (configHooks: NestedHooks<HooksT>) => () => void;
    removeHooks<NameT extends HookNameT>(name: NameT | NameT[]): void;
    removeHooks(configHooks: NestedHooks<HooksT>): void;
    removeAllHooks: () => void;
    /**
     * 同步执行 hooks，并返回最终结果
     */
    callHookSync: <NameT extends HookNameT>(name: NameT, ...args: InferCallbackParams<HooksT, NameT>) => InferCallbackReturn<HooksT, NameT>;
    /**
     * 异步串行执行 hooks，并返回最终结果
     */
    callHook: <NameT extends HookNameT>(name: NameT, ...args: InferCallbackParams<HooksT, NameT>) => Promisify<InferCallbackReturn<HooksT, NameT>>;
    /**
     * 异步并行执行 hooks，并返回结果列表
     */
    callHookParallel: <NameT extends HookNameT>(name: NameT, ...args: InferCallbackParams<HooksT, NameT>) => Promisify<InferCallbackReturn<HooksT, NameT>[]>;
    /**
     * 根据指定 Caller 执行 hooks
     */
    callHookWith: <NameT extends HookNameT, Caller extends HookTaskCaller<NameT, InferCallbackParams<HooksT, NameT>>>(caller: Caller, name: NameT, ...args: InferCallbackParams<HooksT, NameT>) => ReturnType<Caller>;
    beforeEach: (fn: (ctx: HookContext<HooksT[HookNameT]>) => void) => () => void;
    afterEach: (fn: (ctx: HookContext<HooksT[HookNameT]>) => void) => () => void;
    /**
     * 获取 Hook 执行上下文
     *
     * ***注意：仅支持在 Hook 内顶级作用域同步获取！***
     *
     * @param hookName Hook 名称，用于获取指定 hook 的类型提示，推荐传入
     *
     * @example
     * ```ts
     * hooks.hook('test', () => {
     *   const ctx = hooks.getContext('test')!
     *   // => HookContext { ... }
     * })
     * ```
     */
    getContext: <NameT extends HookNameT>(hookName?: NameT) => HookContext<HooksT[NameT]> | null;
}
export declare function createHooks<T extends Record<string, any>>(): Hookable<T>;
